@@ -1,3 +1,7 @@
+import sys
+import traceback as tb
+
+
 def create_exceptions_mapping(*sources):
     """Create a single mapping of exception names to their classes given any number of dictionaries mapping variable
     names to variables e.g. `locals()`, `globals()` or a module. Non-exception variables are filtered out. This function
@@ -19,3 +23,17 @@ def create_exceptions_mapping(*sources):
             continue
 
     return exceptions_mapping
+
+
+def convert_exception_to_primitives():
+    """Convert an exception into a dictionary of its type, message, and traceback as JSON-serialisable primitives.
+
+    :return dict: a dictionary with "type", "message" and "traceback" keys and JSON-serialisable values
+    """
+    exception_info = sys.exc_info()
+
+    return {
+        "type": exception_info[0].__name__,
+        "message": f"{exception_info[1]}",
+        "traceback": tb.format_list(tb.extract_tb(exception_info[2])),
+    }
